<?php

namespace Drupal\graphql_compose_webform\Plugin\GraphQL\SchemaExtension;

use Drupal\graphql\GraphQL\ResolverBuilder;
use Drupal\graphql\GraphQL\ResolverRegistryInterface;
use Drupal\graphql_compose\Plugin\GraphQL\SchemaExtension\ResolverOnlySchemaExtensionPluginBase;
use Drupal\webform\WebformInterface;
use Drupal\webform\WebformSubmissionInterface;

/**
 * Add webform to the Schema.
 *
 * @SchemaExtension(
 *   id = "graphql_compose_webform",
 *   name = "GraphQL Compose Webform",
 *   description = @Translation("Add webform to the Schema."),
 *   schema = "graphql_compose",
 * )
 */
class WebformSchemaExtension extends ResolverOnlySchemaExtensionPluginBase {

  /**
   * {@inheritdoc}
   */
  public function registerResolvers(ResolverRegistryInterface $registry): void {
    $builder = new ResolverBuilder();

    $registry->addFieldResolver(
      'Query',
      'webform',
      $builder->produce('webform_load')
        ->map('id', $builder->fromArgument('id'))
    );

    $this->addWebformFieldResolvers($registry, $builder);
    $this->addMutationFields($registry, $builder);
  }

  /**
   * Register the webform field resolvers.
   *
   * @param \Drupal\graphql\GraphQL\ResolverRegistryInterface $registry
   *   The resolver registry.
   * @param \Drupal\graphql\GraphQL\ResolverBuilder $builder
   *   The resolver builder.
   */
  protected function addWebformFieldResolvers(ResolverRegistryInterface $registry, ResolverBuilder $builder): void {
    $registry->addFieldResolver(
      'Webform',
      'id',
      $builder->callback(fn(WebformInterface $webform) => $webform->id())
    );

    $registry->addFieldResolver(
      'Webform',
      'label',
      $builder->callback(fn(WebformInterface $webform) => $webform->label())
    );

    $registry->addFieldResolver(
      'Webform',
      'description',
      $builder->callback(fn(WebformInterface $webform) => $webform->getDescription())
    );

    $registry->addFieldResolver(
      'Webform',
      'elements',
      $builder->callback(fn(WebformInterface $webform) => $this->resolveWebformElements($webform))
    );
  }

  /**
   * Resolve webform elements.
   *
   * @param \Drupal\webform\WebformInterface $webform
   *   The webform.
   *
   * @return array
   *   The webform elements.
   */
  protected function resolveWebformElements(WebformInterface $webform): array {
    $elements = [];

    foreach ($webform->getElementsInitialized() as $element) {
      $graphql_element = [];
      foreach ($element as $key => $value) {
        if (strpos($key, '#') === 0) {
          $key = substr($key, 1);
        }
        if ($key === 'submit__label') {
          $key = 'title';
        }
        $graphql_element[$key] = $value;
      }

      $graphql_element['options'] = $this->resolveElementOptions($graphql_element);
      $graphql_element['states'] = $this->resolveElementStates($graphql_element);
      $elements[] = $graphql_element;
    }

    return $elements;
  }

  /**
   * Resolve element options.
   *
   * @param array $element
   *   The webform element.
   *
   * @return array
   *   The resolved options.
   */
  protected function resolveElementOptions(array $element): array {
    $options = [];
    if (isset($element['options'])) {
      foreach ($element['options'] as $key => $option) {
        $options[] = [
          'id' => $key,
          'value' => $option,
        ];
      }
    }

    return $options;
  }

  /**
   * Resolve element states.
   *
   * @param array $element
   *   The webform element.
   *
   * @return array
   *   The resolved states.
   */
  protected function resolveElementStates(array $element): array {
    $states = [];
    if (isset($element['states'])) {
      foreach ($element['states'] as $state_type => $state) {
        foreach ($state as $identifier => $value) {
          // Remove the :input[name=" and "] parts from the identifier.
          $element_name = preg_replace('/^:input\[name="(.*)"\]$/', '$1', $identifier);
          if ($element_name === NULL) {
            $element_name = $identifier;
          }
          $states[] = [
            'condition' => $state_type,
            'element' => $element_name,
            'trigger' => $value['value'],
          ];
        }
      }
    }

    return $states;
  }

  /**
   * Register the webform element state fields.
   *
   * @param \Drupal\graphql\GraphQL\ResolverRegistryInterface $registry
   *   The resolver registry.
   * @param \Drupal\graphql\GraphQL\ResolverBuilder $builder
   *   The resolver builder.
   */
  protected function addMutationFields(ResolverRegistryInterface $registry, ResolverBuilder $builder): void {
    $registry->addFieldResolver('Mutation', 'submitWebform', $builder->compose(
      $builder->produce('webform_submit')
        ->map('id', $builder->fromArgument('id'))
        ->map('data', $builder->fromArgument('data'))
    ));

    $registry->addFieldResolver(
      'WebformSubmission',
      'submission',
      $builder->callback(fn(WebformSubmissionInterface $result) => $result)
    );
  }

}
